pub fn intern_count_rows(
feature_names: &[Box<str>],
policy: UnparsedRowPolicy,
) -> CountRowMapExpand description
Intern a count matrix’s feature axis onto a dense gene axis, pairing a gene’s two channel rows under one id.
pub fn intern_count_rows(
feature_names: &[Box<str>],
policy: UnparsedRowPolicy,
) -> CountRowMapIntern a count matrix’s feature axis onto a dense gene axis, pairing a gene’s two channel rows under one id.