pub enum Purpose {
Explore,
Pick {
verb: &'static str,
},
PickBoth {
verb: &'static str,
},
}Expand description
What the explorer is for.
Variants
Explore
Look around (stat --interactive); Tab switches sides.
Pick
Mark entries of the starting side for a command labelled verb;
Enter returns them.
PickBoth
Mark rows and columns for a command labelled verb (Tab switches
sides, each keeping its marks); Enter returns both.