explore

Function explore 

pub fn explore(explorer: StatExplorer<'_>) -> Result<Option<Picked>>
Run the explorer full screen until the user quits. When picking, returns the marked entries if the user finished with Enter.