pub struct Values {
pub names: Vec<Box<str>>,
pub columns: Vec<Vec<f32>>,
}Expand description
The values of some entries of one side against every entry of the other.
Fields
names: Vec<Box<str>>The other side’s names, one per value row.
columns: Vec<Vec<f32>>One column of values per requested entry, in request order.