SparseMtxData

Struct SparseMtxData 

pub struct SparseMtxData { /* private fields */ }
10x-like cell-feature matrix with zarr backend (feature x cell)

(root)
    ├── nrow
    ├── ncell
    ├── by_column
    │   ├── data
    │   ├── indices (row indices)
    │   └── indptr (column pointers)
    └── by_row
        ├── data
        ├── indices (column indices)
        └── indptr (row pointers)

Implementations

Source

impl SparseMtxData

pub fn new(zarr_file: Option<&str>) -> Result<Self>

Create an empty new SparseMtxData instance with a zarr backend file If no backend_file is provided, a temporary file will be created.

  • backend_file - Optional zarr backend file
pub fn open(backend_file: &str) -> Result<Self>

Create SparseMtxData instance from an existing zarr backend file

  • zarr_file - zarr backend file (directory or .zarr.zip)
pub fn from_mtx_file( mtx_file: &str, backend_file: Option<&str>, index_by_row: Option<bool>, ) -> Result<Self>

Create SparseMtxData from mtx file with backend_file as the backend file. If no backend_file is provided, it will be the same as mtx_file with .zarr extension.

  • mtx_file: mtx file to be read into zarr backend
  • backend_file: zarr file to be associated with
  • index_by_row: if true, the matrix will be indexed by row
pub fn from_ndarray( array: &Array2<f32>, zarr_file: Option<&str>, index_by_row: Option<bool>, ) -> Result<Self>

Create a new SparseMtxData instance from an ndarray array

  • array - 2D array to be added to the backend
  • backend_file - Optional zarr backend file
  • index_by_row - Optional flag to index by row (CSR format)
pub fn from_dmatrix( matrix: &DMatrix<f32>, zarr_file: Option<&str>, index_by_row: Option<bool>, ) -> Result<Self>

Create a new SparseMtxData instance from an DMatrix array

  • array - 2D array to be added to the backend
  • backend_file - Optional zarr backend file
  • index_by_row - Optional flag to index by row (CSR format)
pub fn print_hierarchy(&self) -> Result<()>

Show the hierarchy of the zarr store