Module io_helpers

Module io_helpers 

Re-exports

pub use crate::sparse_io::MAX_COLUMN_NAME_IDX;
pub use crate::sparse_io::MAX_ROW_NAME_IDX;

Functions

chunk_elems
Compute the chunk size (in elements) for a 1-D array of nelem elements, each elem_bytes wide. Returns at least 1 even for empty arrays.
parse_index_spec
Parse an index specification into an explicit, ordered list of indices.
parse_name_file
Parse a names file into one joined string per line.
read_col_names
Read column names from a file, joining multi-column names with COLUMN_SEP
read_row_names
Read row names from a file, joining multi-column names with ROW_SEP