pub type ZarrStoreRw = (Arc<dyn ZReadStorageTraits>, Option<Arc<FilesystemStore>>);Expand description
A read store paired with an optional write store (present for directories, None for zips).
pub type ZarrStoreRw = (Arc<dyn ZReadStorageTraits>, Option<Arc<FilesystemStore>>);A read store paired with an optional write store (present for directories, None for zips).