Structs
Functions
- collect_
column_ stat - collect column-wise sufficient statistics for Q/C
- collect_
column_ stat_ across_ vec - collect column-wise sufficient statistics for Q/C
- collect_
row_ stat - collect row-wise sufficient statistics for Q/C
- collect_
row_ stat_ across_ vec - collect row-wise sufficient statistics for Q/C
- collect_
stratified_ row_ stat_ across_ vec - collect row statistics for each group of columns
- print_
nnz_ summary - Print summary statistics + an ASCII log10(x+1) histogram of a per-row or
per-column statistic vector (
metricnames it, e.g. “nnz”, “sum”, “mean”). A non-zerocutoffmarks the cutoff bin and reports how much it removes; an optionalsuggestedvalue reports the trough suggestion. - squeeze_
by_ nnz - squeeze out rows and columns with excessive zero values
- suggest_
nnz_ cutoff - Suggest an nnz cutoff at the deepest trough of the nnz distribution,
the gap between the ambient (empty barcode) peak and the cell peak.
Columns with
nnz >= cutoffare kept.